@@ -21,12 +21,19 @@
 #include "selint_error.h"
 #include "selint_config.h"
 
+enum extra_ids {
+	X_ID_UNUSED_IF  = 1,
+	X_ID_EXCL_AV    = 2,
+	X_END
+};
+
 enum convention_ids {
 	C_ID_TE_ORDER       = 1,
 	C_ID_IF_COMMENT     = 4,
 	C_ID_UNORDERED_PERM = 5,
 	C_ID_UNORDERED_REQ  = 6,
 	C_ID_SELF           = 7,
+	C_ID_FOREIGN_CONDID = 8,
 	C_END
 };
 
@@ -56,6 +63,7 @@ enum warn_ids {
 	W_ID_MOD_NAME_FILE     = 9,
 	W_ID_UNKNOWN_CALL      = 10,
 	W_ID_IF_DECL_NOT_OWN   = 11,
+	W_ID_UNKNOWN_COND_ID   = 12,
 	W_END
 };
 
@@ -85,6 +93,7 @@ enum file_flavor {
 
 struct check_data {
 	char *mod_name;
+	const char *filepath;
 	char *filename;
 	enum file_flavor flavor;
 	const struct config_check_data *config_check_data;
@@ -115,6 +124,8 @@ struct checks {
 extern int found_issue;
 // Whether found issues are printed individually
 extern int suppress_output;
+// Whether to print full paths
+extern int full_path;
 
 /*********************************************
 * Add an check to be called on check_flavor nodes
