@@ -30,6 +30,7 @@
 #include "selint_config.h"
 #include "startup.h"
 #include "color.h"
+#include "xalloc.h"
 
 // ASCII characters go up to 127
 #define CONTEXT_ID          128
@@ -37,6 +38,7 @@
 #define SUMMARY_ONLY_ID     130
 #define SCAN_HIDDEN_DIRS_ID 131
 #define DEBUG_PARSER_ID     132
+#define FULL_PATH_ID        133
 
 extern int yydebug;
 
@@ -63,6 +65,7 @@ static void usage(void)
 		"  -e, --enable=CHECKID\t\tEnable check with the given ID.\n"\
 		"  -E, --only-enabled\t\tOnly run checks that are explicitly enabled with\n"\
 		"\t\t\t\tthe --enable option.\n"\
+		"      --full-path\t\tPrint full path for files.\n"\
 		"  -F, --fail\t\t\tExit with a non-zero value if any issue was found.\n"\
 		"  -h, --help\t\t\tDisplay this menu.\n"\
 		"  -l, --level=LEVEL\t\tOnly list errors with a severity level at or\n"\
@@ -124,6 +127,7 @@ int main(int argc, char **argv)
 			{ "disable",          required_argument, NULL,          'd' },
 			{ "enable",           required_argument, NULL,          'e' },
 			{ "fail",             no_argument,       NULL,          'F' },
+			{ "full-path",        no_argument,       NULL,          FULL_PATH_ID },
 			{ "only-enabled",     no_argument,       NULL,          'E' },
 			{ "help",             no_argument,       NULL,          'h' },
 			{ "level",            required_argument, NULL,          'l' },
@@ -195,28 +199,28 @@ int main(int argc, char **argv)
 			// Disable a given check
 			if (cl_d_cursor) {
 				cl_d_cursor->next =
-					calloc(1, sizeof(struct string_list));
+					xcalloc(1, sizeof(struct string_list));
 				cl_d_cursor = cl_d_cursor->next;
 			} else {
 				cl_d_cursor =
-					calloc(1, sizeof(struct string_list));
+					xcalloc(1, sizeof(struct string_list));
 				cl_disabled_checks = cl_d_cursor;
 			}
-			cl_d_cursor->string = strdup(optarg);
+			cl_d_cursor->string = xstrdup(optarg);
 			break;
 
 		case 'e':
 			// Enable a given check
 			if (cl_e_cursor) {
 				cl_e_cursor->next =
-					calloc(1, sizeof(struct string_list));
+					xcalloc(1, sizeof(struct string_list));
 				cl_e_cursor = cl_e_cursor->next;
 			} else {
 				cl_e_cursor =
-					calloc(1, sizeof(struct string_list));
+					xcalloc(1, sizeof(struct string_list));
 				cl_enabled_checks = cl_e_cursor;
 			}
-			cl_e_cursor->string = strdup(optarg);
+			cl_e_cursor->string = xstrdup(optarg);
 			break;
 
 		case 'E':
@@ -224,6 +228,11 @@ int main(int argc, char **argv)
 			only_enabled = 1;
 			break;
 
+		case FULL_PATH_ID:
+			// Display full paths for files
+			full_path = true;
+			break;
+
 		case 'F':
 			// Exit non-zero if any issue was found
 			fail_on_finding = 1;
@@ -364,27 +373,27 @@ int main(int argc, char **argv)
 
 	print_if_verbose("Severity level set to %c\n", severity);
 
-	if (optind == argc) {
+	if (optind >= argc) {
 		usage();
 		exit(EX_USAGE);
 	}
 
 	struct policy_file_list *te_files =
-		calloc(1, sizeof(struct policy_file_list));
+		xcalloc(1, sizeof(struct policy_file_list));
 
 	struct policy_file_list *if_files =
-		calloc(1, sizeof(struct policy_file_list));
+		xcalloc(1, sizeof(struct policy_file_list));
 
 	struct policy_file_list *fc_files =
-		calloc(1, sizeof(struct policy_file_list));
+		xcalloc(1, sizeof(struct policy_file_list));
 
 	struct policy_file_list *context_te_files =
-		calloc(1, sizeof(struct policy_file_list));
+		xcalloc(1, sizeof(struct policy_file_list));
 
 	struct policy_file_list *context_if_files =
-		calloc(1, sizeof(struct policy_file_list));
+		xcalloc(1, sizeof(struct policy_file_list));
 
-	char **paths = malloc(sizeof(char *) * (unsigned)argc - (unsigned)optind + 2);
+	char **paths = xmalloc(sizeof(char *) * (unsigned)argc - (unsigned)optind + 2);
 
 	int i = 0;
 	while (optind < argc) {
@@ -401,6 +410,7 @@ int main(int argc, char **argv)
 	char *modules_conf_path = NULL;
 	char *obj_perm_sets_path = NULL;
 	char *access_vector_path = NULL;
+	struct string_list *global_cond_files = NULL;
 
 	while (file) {
 		const char *suffix = (file->fts_pathlen > 3) ? (file->fts_path + file->fts_pathlen - 3) : NULL;
@@ -413,7 +423,7 @@ int main(int argc, char **argv)
 			file_list_push_back(if_files,
 			                    make_policy_file(file->fts_path,
 			                                     NULL));
-			char *mod_name = strdup(file->fts_name);
+			char *mod_name = xstrdup(file->fts_name);
 			mod_name[file->fts_namelen - 3] = '\0';
 			insert_into_mod_layers_map(mod_name, file->fts_parent->fts_name);
 			free(mod_name);
@@ -424,15 +434,19 @@ int main(int argc, char **argv)
 		} else if (source_flag
 		           && !strcmp(file->fts_name, "modules.conf")) {
 			// TODO: Make modules.conf name configurable
-			modules_conf_path = strdup(file->fts_path);
+			modules_conf_path = xstrdup(file->fts_path);
 		} else if (source_flag
 		           && !strcmp(file->fts_name, "obj_perm_sets.spt")) {
 			// TODO: Make obj_perm_sets.spt name configurable
-			obj_perm_sets_path = strdup(file->fts_path);
+			obj_perm_sets_path = xstrdup(file->fts_path);
 		} else if (source_flag
 		           && !strcmp(file->fts_name, "access_vectors")) {
 			// TODO: Make access_vectors name configurable
-			access_vector_path = strdup(file->fts_path);
+			access_vector_path = xstrdup(file->fts_path);
+		} else if (source_flag
+		           && (!strcmp(file->fts_name, "global_booleans") || !strcmp(file->fts_name, "global_tunables"))) {
+			// TODO: Make names configurable
+			global_cond_files = concat_string_lists(global_cond_files, sl_from_str(file->fts_path));
 		} else {
 			// Directories might get traversed twice: preorder and final visit.
 			// Print only the final visit
@@ -491,15 +505,19 @@ int main(int argc, char **argv)
 			} else if (source_flag
                                    && !modules_conf_path
                                    && 0 == strcmp(file->fts_name, "modules.conf")) {
-				modules_conf_path = strdup(file->fts_path);
+				modules_conf_path = xstrdup(file->fts_path);
 			} else if (source_flag
                                    && !obj_perm_sets_path
                                    && 0 == strcmp(file->fts_name, "obj_perm_sets.spt")) {
-				obj_perm_sets_path = strdup(file->fts_path);
+				obj_perm_sets_path = xstrdup(file->fts_path);
 			} else if (source_flag
                                    && !access_vector_path
                                    && 0 == strcmp(file->fts_name, "access_vectors")) {
-				access_vector_path = strdup(file->fts_path);
+				access_vector_path = xstrdup(file->fts_path);
+			} else if (source_flag
+			           && !str_in_sl(file->fts_path, global_cond_files)
+			           && (0 == strcmp(file->fts_name, "global_booleans") || 0 == strcmp(file->fts_name, "global_tunables"))) {
+				global_cond_files = concat_string_lists(global_cond_files, sl_from_str(file->fts_path));
 			}
 			file = fts_read(ftsp);
 		}
@@ -550,6 +568,17 @@ int main(int argc, char **argv)
 			printf("%sWarning%s: Failed to locate obj_perm_sets.spt file.\n", color_warning(), color_reset());
 		}
 
+		if (global_cond_files) {
+			enum selint_error res = load_global_conditions(global_cond_files);
+			if (res != SELINT_SUCCESS) {
+				printf("%sWarning%s: Failed to parse global conditions: %d\n", color_warning(), color_reset(), res);
+			} else {
+				print_if_verbose("Loaded global conditions\n");
+			}
+		} else {
+			printf("%sWarning%s: Failed to locate global conditions files.\n", color_warning(), color_reset());
+		}
+
 	} else {
 		enum selint_error r = load_access_vectors_kernel("/sys/fs/selinux/class");
 		if (r != SELINT_SUCCESS) {
@@ -587,12 +616,14 @@ int main(int argc, char **argv)
 		free(obj_perm_sets_path);
 		free(access_vector_path);
 		free(modules_conf_path);
+		free_string_list(global_cond_files);
 		return EX_CONFIG;
 	}
 
 	free(obj_perm_sets_path);
 	free(access_vector_path);
 	free(modules_conf_path);
+	free_string_list(global_cond_files);
 
 	enum selint_error res = run_analysis(ck, te_files, if_files, fc_files, context_te_files, context_if_files, custom_fc_macros, &ccd);
 	switch (res) {
