@@ -24,6 +24,57 @@
 #include "ordering.h"
 #include "util.h"
 #include "perm_macro.h"
+#include "xalloc.h"
+
+struct check_result *check_excluding_av_rule(__attribute__((unused)) const struct check_data *data,
+                                             const struct policy_node *node)
+{
+	if (node->flavor == NODE_AV_RULE) {
+		// ignore neverallowallow rules
+		if (node->data.av_data->flavor == AV_RULE_NEVERALLOW) {
+			return NULL;
+		}
+
+		for (const struct string_list *sources = node->data.av_data->sources; sources; sources = sources->next) {
+			if (sources->string[0] == '-') {
+				return make_check_result('X', X_ID_EXCL_AV,
+							"AV rule with excluded source");
+			}
+		}
+
+		for (const struct string_list *targets = node->data.av_data->targets; targets; targets = targets->next) {
+			if (targets->string[0] == '-') {
+				return make_check_result('X', X_ID_EXCL_AV,
+							"AV rule with excluded target");
+			}
+		}
+
+		return NULL;
+	}
+
+	if (node->flavor == NODE_IF_CALL) {
+		const struct string_list *args = node->data.ic_data->args;
+		bool check_current = false;
+
+		for (; args; args = args->next) {
+			// only check if the first section argument is a type or attribute
+			if (args->arg_start) {
+				check_current = look_up_in_decl_map(args->string, DECL_TYPE) || look_up_in_decl_map(args->string, DECL_ATTRIBUTE);
+				continue;
+			}
+
+			if (check_current && args->string[0] == '-' &&
+			    (look_up_in_decl_map(&args->string[1], DECL_TYPE) || look_up_in_decl_map(&args->string[1], DECL_ATTRIBUTE))) {
+				return make_check_result('X', X_ID_EXCL_AV,
+							 "Interface or macro call with excluded type");
+			}
+		}
+
+		return NULL;
+	}
+
+	return alloc_internal_error("Invalid node type for `check_excluding_av_rule`");
+}
 
 struct check_result *check_te_order(const struct check_data *data,
                                     const struct policy_node *node)
@@ -170,6 +221,44 @@ struct check_result *check_no_self(__attribute__((unused)) const struct check_da
 
 }
 
+struct check_result *check_foreign_cond_id(const struct check_data
+                                           *data,
+                                           const struct policy_node
+                                           *node)
+{
+	if (node->flavor != NODE_BOOLEAN_POLICY && node->flavor != NODE_TUNABLE_POLICY) {
+		return alloc_internal_error("Invalid node type for `check_foreign_cond_id`");
+	}
+
+	const struct string_list *identifiers = node->data.cd_data->identifiers;
+
+	for (const struct string_list *ids = identifiers; ids; ids = ids->next) {
+		const char *id = ids->string;
+
+		const char *mod_name = look_up_in_decl_map(id, DECL_BOOL);
+
+		/* Ignore non-existent identifiers, covered by W-012 */
+		if (mod_name == NULL) {
+			continue;
+		}
+
+		if (0 == strcmp(mod_name, data->mod_name)) {
+			continue;
+		}
+
+		if (0 == strcmp(mod_name, "__global__")) {
+			continue;
+		}
+
+		return make_check_result('C', C_ID_FOREIGN_CONDID,
+		                         "Identifier %s in expression for conditional block not found in own module, but in module %s (candidate for global declaration or interface)",
+		                         id,
+		                         mod_name);
+        }
+
+	return NULL;
+}
+
 struct check_result *check_require_block(const struct check_data *data,
                                          const struct policy_node *node)
 {
@@ -620,7 +709,7 @@ struct check_result *check_module_file_name_mismatch(const struct check_data
 static bool starts_with_module_prefix(const char *name)
 {
 	for (const char *prefix = strchr(name, '_'); prefix; prefix = strchr(prefix + 1, '_')) {
-		char *search_mod = strndup(name, (size_t)(prefix - name));
+		char *search_mod = xstrndup(name, (size_t)(prefix - name));
 		if (look_up_in_mods_map(search_mod)) {
 			free(search_mod);
 			return true;
@@ -669,6 +758,78 @@ struct check_result *check_unknown_interface_call(__attribute__((unused)) const
 				 if_name);
 }
 
+/* Check whether the name is a simple parameter, like $1. Combined ones like $1_t are not considered. */
+static bool is_bare_parameter(const char *name)
+{
+	if (name[0] != '$' || name[1] == '\0') {
+		return false;
+	}
+
+	for (const char *c = name + 1; *c; c++) {
+		if (!isdigit((unsigned char)*c)) {
+			return false;
+		}
+	}
+
+	return true;
+}
+
+static bool temp_has_bool_decl(const char *temp_name, const char *bool_name)
+{
+	if (temp_name == NULL) {
+		return false;
+	}
+
+	for (const struct decl_list *dl = look_up_decl_in_template_map(temp_name); dl; dl = dl->next) {
+		const struct declaration_data *decl = dl->decl;
+
+		if (decl->flavor != DECL_BOOL) {
+			continue;
+		}
+
+		if (0 == strcmp(decl->name, bool_name)) {
+			return true;
+		}
+	}
+
+	return false;
+}
+
+struct check_result *check_unknown_cond_id(__attribute__((unused)) const struct check_data
+					   *data,
+					   const struct policy_node
+					   *node)
+{
+	if (node->flavor != NODE_BOOLEAN_POLICY && node->flavor != NODE_TUNABLE_POLICY) {
+		return alloc_internal_error("Invalid node type for `check_foreign_cond_id`");
+	}
+
+	const struct string_list *identifiers = node->data.cd_data->identifiers;
+
+	for (const struct string_list *ids = identifiers; ids; ids = ids->next) {
+		const char *id = ids->string;
+
+		/* Ignore interface parameters */
+		if (is_bare_parameter(id)) {
+			continue;
+		}
+
+		if (look_up_in_decl_map(id, DECL_BOOL) != NULL) {
+			continue;
+		}
+
+		if (temp_has_bool_decl(get_name_if_in_template(node), id)) {
+			continue;
+		}
+
+		return make_check_result('W', W_ID_UNKNOWN_COND_ID,
+					 "Expression for conditional block uses unknown identifier %s",
+					 id);
+	}
+
+	return NULL;
+}
+
 struct check_result *check_declaration_interface_nameclash(__attribute__((unused)) const struct check_data
 							   *data,
 							   const struct policy_node
@@ -685,7 +846,7 @@ struct check_result *check_declaration_interface_nameclash(__attribute__((unused
 	return NULL;
 }
 
-bool check_unknown_permission_condition()
+bool check_unknown_permission_condition(void)
 {
 	// ignore if no permission or permission macro have been parsed
 	if (permmacros_map_count() == 0) {
@@ -737,7 +898,7 @@ struct check_result *check_unknown_permission(__attribute__((unused)) const stru
 	return NULL;
 }
 
-bool check_unknown_class_condition()
+bool check_unknown_class_condition(void)
 {
 	// ignore if no class has been parsed
 	if (decl_map_count(DECL_CLASS) == 0) {
