@@ -19,6 +19,16 @@
 
 #include "check_hooks.h"
 
+/*********************************************
+ * Check for access vector rules with excluded source or target.
+ * Called on NODE_AV_RULE and NODE_IF_CALL nodes.
+ * data - metadata about the file
+ * node - the node to check
+ * returns NULL if passed or check_result for issue X-002
+ *********************************************/
+struct check_result *check_excluding_av_rule(const struct check_data *data,
+                                             const struct policy_node *node);
+
 /*********************************************
 * Check for violations of te file ordering conventions.
 * The refpolicy conventions (which is all that can be checked as of now are
@@ -58,6 +68,18 @@ struct check_result *check_unordered_perms(const struct check_data *data,
 struct check_result *check_no_self(const struct check_data *data,
                                    const struct policy_node *node);
 
+/*********************************************
+ * Check for identifiers in conditional expressions not declared in own module.
+ * Called on NODE_BOOLEAN_POLICY and NODE_TUNABLE_POLICY nodes.
+ * data - metadata about the file currently being scanned
+ * node - the node to check
+ * returns NULL if passed or check_result for issue C-008
+*********************************************/
+struct check_result *check_foreign_cond_id(const struct check_data
+                                           *data,
+                                           const struct policy_node
+                                           *node);
+
 /*********************************************
 * Check for the presence of require blocks in TE files.
 * Interface calls are to be preferred.
@@ -196,6 +218,18 @@ struct check_result *check_unknown_interface_call(const struct check_data
                                                   const struct policy_node
                                                   *node);
 
+/*********************************************
+ * Check for unknown identifiers in conditional expressions.
+ * Called on NODE_BOOLEAN_POLICY and NODE_TUNABLE_POLICY nodes.
+ * data - metadata about the file currently being scanned
+ * node - the node to check
+ * returns NULL if passed or check_result for issue W-012
+*********************************************/
+struct check_result *check_unknown_cond_id(const struct check_data
+                                           *data,
+                                           const struct policy_node
+                                           *node);
+
 /*********************************************
  * Check for clash of declaration and interface names.
  * This will cause macro expansion to enter an endless loop
